<?php

namespace Drupal\simple_survey\Form;

use Drupal\Core\Entity\EntityConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\simple_survey\Service\ResponseExportService;

/**
 * CSV Export Confirmation form.
 */
class ConfirmCsvExportForm extends EntityConfirmFormBase {

  /**
   * {@inheritDoc}
   */
  public function getQuestion() {
    return $this->t('Are you sure you want to export the responses for @survey to CSV.', [
      '@survey' => $this->entity->label(),
    ]);
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $survey = $this->entity->id();
    $options = [
      'survey' => $survey,
      'export_type' => 'csv',
      'sent_to_bq' => FALSE,
      'truncate' => FALSE,
      'batch_size' => 75,
      'filename' => NULL,
      'filetime' => NULL,
      'file_path' => NULL,
      'chunk' => 0,
    ];

    $options = ResponseExportService::createCsvFile($options);

    $retrieved_responses = ResponseExportService::retrieveResponses($options);
    $options['total'] = count($retrieved_responses);

    $response_chunks = array_chunk($retrieved_responses, $options['batch_size']);

    $operations = [];

    foreach ($response_chunks as $rc) {
      $operations[] = [
        '\Drupal\simple_survey\Service\ResponseExportService::processOptionsData',
        [
          $options,
          $rc,
        ],
      ];
      $options['chunk']++;
    }

    $batch = [
      'title' => $this->t('Creating CSV export :filename for :survey', [
        ':survey' => $this->entity->label(),
        ':filename' => $options['filename'],
      ]),
      'operations' => $operations,
      'finished' => '\Drupal\simple_survey\Service\ResponseExportService::finishedFileProcessing',
      'init_message' => $this->t('Exporting data to CSV for @source', [
        '@source' => $this->entity->label(),
      ]),
      'progress_message' => $this->t('Time left: @estimate | Time elapsed: @elapsed', [
        '@source' => $this->entity->label(),
      ]),
      'error_message' => $this->t('The export progress has encountered an error.'),
    ];

    batch_set($batch);

    $form_state->setRedirect('entity.simple_survey.edit_form', [
      'simple_survey' => $survey,
    ]);
  }

  /**
   * {@inheritDoc}
   */
  public function getCancelUrl() {
    return Url::fromRoute('entity.simple_survey.edit_form', [
      'simple_survey' => $this->entity->id(),
    ]);
  }

  /**
   * {@inheritDoc}
   */
  public function getConfirmText() {
    return $this->t('Export to CSV');
  }

  /**
   * {@inheritDoc}
   */
  public function getDescription() {
    $message = $this->t('You are exporting responses to CSV for @survey. Do you wish to continue?', [
      '@survey' => $this->entity->label(),
    ]);

    return $message;
  }

}
